import { Component } from 'vue';
import { CSSProperties } from 'vue';
import { DefineComponent } from 'vue';
import { InjectionKey } from 'vue';
import { Ref } from 'vue';
import { VNodeProps } from 'vue';

export declare type ComponentType = Component | DefineComponent;

export declare interface Field {
    name: string;
    props: Props;
    slots?: Recordable<() => void>;
    on?: Recordable<string | ((event: any, $inject: InjectionType, ref: Ref<any>, field: Field) => void) | undefined>;
}

export declare type Hook<T extends Field = Field> = (field: T, inject?: InjectionType) => void;

export declare type InjectionType = {
    readonly?: boolean;
    rootRef?: Ref<any>;
    [key: string]: any;
};

export declare interface Options {
    type: 'string' | 'boolean' | 'number';
    value: any;
    label: string;
    disabled?: boolean;
    children?: Options[];
}

export declare interface Props extends VNodeProps {
    style?: CSSProperties;
    class?: string[];
    options?: Options[];
    [key: string]: any;
}

export declare type Recordable<T = any> = Record<string, T>;

export declare const RENDER_INJECTION_KEY: InjectionKey<string>;

export declare interface RenderContext<T extends Field = Field> {
    Render: ComponentType;
    addComponent: (name: string, component: ComponentType, hook?: Hook<T>) => void;
    removeComponent: (name: string) => void;
}

export declare const useRender: <T extends Field = Field>(globalHook?: Hook<T>) => RenderContext<T>;

export { }
